import re
import unittest2
import httpretty
import json
import pandas
import numpy
import six
from quandl.model.data import Data
from quandl.model.datatable import Datatable
from mock import patch, call
from test.factories.datatable_data import DatatableDataFactory
from test.factories.datatable_meta import DatatableMetaFactory


class DatatableDataTest(unittest2.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.expected_column_names = [six.u('per_end_date'),
                                     six.u('ticker'),
                                     six.u('tot_oper_exp')]
        cls.expected_column_types = [six.u('Date'),
                                     six.u('String'),
                                     six.u('String')]
        cls.data_object = Data(['2015-07-11', 'AAPL', 440.0],
                               meta={'columns': cls.expected_column_names,
                                     'column_types': cls.expected_column_types})

    def test_to_pandas_returns_pandas_dataframe_object(self):
        data = self.data_object.to_pandas()
        self.assertIsInstance(data, pandas.core.frame.DataFrame)

    # don't set dataFrame for datatable
    def test_pandas_dataframe_index_is_datetime(self):
        df = self.data_object.to_pandas()
        self.assertEqual(df.index.name, 'None')

    def test_to_numpys_returns_numpy_object(self):
        data = self.data_object.to_numpy()
        self.assertIsInstance(data, numpy.core.records.recarray)

    def test_to_csv_returns_expected_csv(self):
        data = self.data_object.to_csv()
        expected = "None,per_end_date,ticker,tot_oper_exp\n0,2015-07-11,AAPL,440.0\n"
        self.assertEqual(data, expected)

    def test_column_names(self):
        actual = self.data_object.columns
        self.assertEqual(actual, self.expected_column_names)

    def test_meta_exists(self):
        self.assertIsNotNone(self.data_object.meta)


class ListDatatableDataTest(unittest2.TestCase):

    @classmethod
    def setUpClass(cls):
        httpretty.enable()
        datatable_data = {'datatable': DatatableDataFactory.build()}
        meta = {'meta': DatatableMetaFactory.build()}
        datatable_data.update(meta)
        httpretty.register_uri(httpretty.GET,
                               re.compile(
                                   'https://www.quandl.com/api/v3/datatables/*'),
                               body=json.dumps(datatable_data))
        cls.expected_raw_data = []
        cls.expected_list_values = []

    @classmethod
    def tearDownClass(cls):
        httpretty.disable()
        httpretty.reset()

    @patch('quandl.connection.Connection.request')
    def test_data_calls_connection(self, mock):
        datatable = Datatable('ZACKS/FC')
        Data.page(datatable, params={'ticker': ['AAPL', 'MSFT'],
                                     'per_end_date': {'gte': {'2015-01-01'}},
                                     'qopts': {'columns': ['ticker', 'per_end_date']}})
        expected = call('get', 'datatables/ZACKS/FC', params={'ticker': ['AAPL', 'MSFT'],
                                                              'per_end_date':
                                                              {'gte': {'2015-01-01'}},
                                                              'qopts': {'columns':
                                                                        ['ticker',
                                                                         'per_end_date']}})
        self.assertEqual(mock.call_args, expected)

    def test_values_and_meta_exist(self):
        datatable = Datatable('ZACKS/FC')
        results = Data.page(datatable, params={})
        self.assertIsNotNone(results.values)
        self.assertIsNotNone(results.meta)

    def test_to_pandas_returns_pandas_dataframe_object(self):
        datatable = Datatable('ZACKS/FC')
        results = Data.page(datatable, params={})
        df = results.to_pandas()
        self.assertIsInstance(df, pandas.core.frame.DataFrame)

    # no index is set for datatable.to_pandas
    def test_pandas_dataframe_index_is_none(self):
        datatable = Datatable('ZACKS/FC')
        results = Data.page(datatable, params={})
        df = results.to_pandas()
        self.assertEqual(df.index.name, 'None')

    # if datatable has Date field then it should be convert to pandas datetime
    def test_pandas_dataframe_date_field_is_datetime(self):
        datatable = Datatable('ZACKS/FC')
        results = Data.page(datatable, params={})
        df = results.to_pandas()
        self.assertIsInstance(df['per_end_date'][0], pandas.datetime)
        self.assertIsInstance(df['per_end_date'][1], pandas.datetime)
        self.assertIsInstance(df['per_end_date'][2], pandas.datetime)
        self.assertIsInstance(df['per_end_date'][3], pandas.datetime)

    def test_to_numpy_returns_numpy_object(self):
        datatable = Datatable('ZACKS/FC')
        results = Data.page(datatable, params={})
        data = results.to_numpy()
        self.assertIsInstance(data, numpy.core.records.recarray)

    def test_to_csv_returns_expected_csv(self):
        datatable = Datatable('ZACKS/FC')
        results = Data.page(datatable, params={})
        data = results.to_csv()
        expected = "None,per_end_date,ticker,tot_oper_exp\n" + \
                   "0,2015-07-11,AAPL,456.9\n" + \
                   "1,2015-07-13,433.3,\n" + \
                   "2,2015-07-14,AAPL,419.1\n" + \
                   "3,2015-07-15,476.5,\n"
        self.assertEqual(data, expected)



class Message:
    ERROR_ARGUMENTS_LIST_FORMAT = 'Your data set must be specified as a string that contains\
        a Quandl code or as a tuple'
    ERROR_AUTHTOKEN_NOT_SUPPORTED = 'The parameter authtoken is no longer supported. \
        For more information please see \
        https://github.com/quandl/quandl-python/blob/master/README.md'
    ERROR_COLUMNS_DATA_NOT_MATCHED = 'The number of columns requested does not match \
        the data returned.'
    ERROR_COLUMN_INDEX_TYPE = 'The column index must be expressed as an integer for %s.'
    ERROR_COLUMN_INDEX_LIST = '%s : column_index must be expressed as a list of \
        integer indexes.'
    ERROR_DATASET_FORMAT = 'Your dataset must either be specified as a string that contains a \
        single Quandl code or an array of Quandl codes'
    ERROR_DATASETS_CODE_MUST_IN_LIST = 'dataset codes must be specified in a list'
    ERROR_FOLDER_ISSUE = 'The folder path specified is incorrect or you do not have \
        permission to access to the folder. Check your settings and try again.'
    ERROR_INVALID_DATABASE_CODE_FROMAT = 'Invalid format used for Quandl database code. \
        The correct format is: `DATABASE_CODE/DATASET_CODE`'
    ERROR_INVALID_DATASET = 'Invalid dataset. Your data set must be specified as a string that \
        contains a Quandl code or as a tuple with multiple Quandl codes'
    ERROR_REQUESTED_INDEX_OUT_OF_RANGE = '%s : The requested index %s is out of range. The \
        minimum index is 1 and the maximum index is %s'
    ERROR_REQUESTED_COLUMN_NOT_EXIST = 'Requested column index %s does not exist'

    WARN_DATA_LIMIT_EXCEEDED = 'This call exceeds the amount of data that quandl.get_table() allows. \
        Please use the following link in your browser, which will download the full results as \
        a CSV file: https://www.quandl.com/api/v3/datatables/%s?qopts.export=true&api_key=%s. See \
        our API documentation for more info: \
        https://docs.quandl.com/docs/in-depth-usage-1#section-download-an-entire-table'
    WARN_PAGE_LIMIT_EXCEEDED = 'To request more pages, please set paginate=true in your \
        quandl.get_table() call. For more information see our documentation: \
        https://github.com/quandl/quandl-python/blob/master/FOR_ANALYSTS.md#things-to-note'
    WARN_PARAMS_NOT_SUPPORTED = '%s will no longer supported. Please use %s instead'
